/** \mainpage Presentation
* Metronome.java is the main class. When Start is pressed, it calls Clicker.java, ClickerStructure.java or ClickerPolyrhythms.java depending on the mode chosen. The sounds arrays are initialisd through Signal. Then the main loop is run in a separate thread.<br>
* MenuSettings.java uses SharedPreferences so the 3 clickers classes can set the audioTrack and their signal from it.<br>
* MenuStructures.java and MenuPolyrhythms.java exchange data wich Metronome.java through intents.<br>
* ContainerStructures|Polyrhythms.java is a commodity class that encapsulates all the data for structures (resp. polyrhythms) and deals with I/O (through intent and disk r/w).
*/

/** \file Metronome.java
* Main class.
*/

package com.arnaud.metronome;

import android.app.Activity;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.view.WindowManager;

import android.media.AudioManager; //Volume control through hardware button
import android.content.Intent; //Pour ouvrir d'autres activity: settings/pattern
import android.widget.Toast; //Pour afficher des trucs
import android.os.SystemClock; //Pour chronometrer le Tap In
import java.util.List;
import java.util.ArrayList;

/*Menu*/
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
/*UI*/
import android.view.View;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.ArrayAdapter;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ImageButton;

//import android.util.Log;

/** Main class
*/
public class Metronome extends Activity
{
	private long lastTapIn; /**< needed to compute the bpm */
	private int mode; /**< Metronome current state. -1: stopped, 0: normal clicker, >0: structure clicker */
	private ContainerStructures containerStructures;
	private ContainerPolyrhythms containerPolyrhythms;
	private ContainerRandomMute containerRandomMute;
	 
	private Clicker clicker;
	private ClickerRandomMute clickerRandomMute;
	private ClickerPolyrhythms clickerPolyrhythms;
	private ClickerStructure clickerStructure;
	
	private EditText editTextBpm;
	private EditText editTextBar;
	private ImageButton imageButtonStartStop;
	private Spinner spinnerMode;
	List<String> listMode = new ArrayList<String>();
	/* Constantes */
	/* Les IDs pour le menu */
	private final int ID_MENU_SETTINGS = 1;
	private final int ID_MENU_STRUCTURES = 2;
	private final int ID_MENU_POLYRHYTHMS = 3;
	private final int ID_MENU_RANDOMMUTE = 4;
	private final int ID_MENU_ABOUT = 5;
	/* Run Mode */
	private final int NO_TASK = -1; /**< Paused */
	private final int CLICKER_TASK = 0; /**< Basic mode. Clicker.java used. */
	private final int RANDOMMUTE_TASK = 1;
	private final int POLYRHYTHMS_TASK = 2; /**< Polyrhythms mode. Several Clicker.java used. If >1 it means structures. */
	/* Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		//Build UI and enable volume control
		editTextBpm = (EditText) findViewById(R.id.editTextBpm);
		editTextBar = (EditText) findViewById(R.id.editTextBar);
		imageButtonStartStop = (ImageButton) findViewById(R.id.imageButtonStartStop);

		spinnerMode = (Spinner) findViewById(R.id.spinnerMode);
		ContainerStructures tmp = new ContainerStructures();
		containerStructures = tmp.loadFromInternal(this);
		ContainerPolyrhythms tmp2 = new ContainerPolyrhythms();
		containerPolyrhythms = tmp2.loadFromInternal(this);
		ContainerRandomMute tmp3 = new ContainerRandomMute();
		containerRandomMute = tmp3.loadFromInternal(this);
		
		int structuresCount = containerStructures.structures.length;
		listMode.add(getResources().getString(R.string.listMode_normal));
		listMode.add(getResources().getString(R.string.listMode_randomMute));
		listMode.add(getResources().getString(R.string.listMode_polyrhythms));
		for (int i=1;i<structuresCount+1;i++)
			listMode.add(String.format(getResources().getString(R.string.listMode_structure),i));
		ArrayAdapter<String> adapterSpinnerMode = new ArrayAdapter<String>(this,android.R.layout.simple_spinner_item,listMode);
		adapterSpinnerMode.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
		spinnerMode.setAdapter(adapterSpinnerMode);
		spinnerMode.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
			@Override
			public void onItemSelected(AdapterView<?> parent, View v, int position, long id) {
				if (position > RANDOMMUTE_TASK) {
					editTextBpm.setText("100");
					editTextBar.setText("0");
					Toast.makeText(getApplicationContext(),R.string.toastMode,Toast.LENGTH_LONG).show();
				}
				else {
					editTextBpm.setText("120");
					editTextBar.setText("4.0");
					//Toast.makeText(getApplicationContext(),"BPM is back to normal use",Toast.LENGTH_SHORT).show();It shows at startup, not good.
				}
			}
			@Override
			public void onNothingSelected(AdapterView<?> arg0) {
			}
		});
	
		setVolumeControlStream(AudioManager.STREAM_MUSIC);
		SharedPreferences settings = getSharedPreferences("settings", 0);
		if (settings.getBoolean("isScreenAlwaysOn",false) == true)
			getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		else
			getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
		spinnerMode.setSelection(settings.getInt("listMode", CLICKER_TASK));
		editTextBpm.setText(String.valueOf(settings.getFloat("bpm", 120)));
		editTextBar.setText(String.valueOf(settings.getFloat("bar", 4)));
		mode = NO_TASK;
	}

	/** Build the menu
	* menu.xml show an empty so we fille it ourself
	*/
	@Override
	public boolean onCreateOptionsMenu (Menu menu)
	{
		/*
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu,menu);
		*/
		
		//menu.add(group.id,item id,order,title)
		menu.add(Menu.NONE,ID_MENU_RANDOMMUTE,Menu.NONE,R.string.menuRandomMute);
		menu.add(Menu.NONE,ID_MENU_POLYRHYTHMS,Menu.NONE,R.string.menuPolyrhythms);
		menu.add(Menu.NONE,ID_MENU_STRUCTURES,Menu.NONE,R.string.menuStructures);
		menu.add(Menu.NONE,ID_MENU_SETTINGS,Menu.NONE,R.string.menuSettings);
		menu.add(Menu.NONE,ID_MENU_ABOUT,Menu.NONE,R.string.menuAbout);
		return true;
	}
	
	/** Set new activities for options, feed them current data */
	@Override
	public boolean onOptionsItemSelected (MenuItem item) {
	switch (item.getItemId()) {
		case ID_MENU_RANDOMMUTE:
			Intent intentMenuRandomMute = new Intent(Metronome.this, MenuRandomMute.class);
			intentMenuRandomMute.putExtra("containerRandomMute",containerRandomMute);
			startActivityForResult(intentMenuRandomMute,ID_MENU_RANDOMMUTE);
			return true;
		case ID_MENU_POLYRHYTHMS:
			Intent intentMenuPolyrhythms = new Intent(Metronome.this, MenuPolyrhythms.class);
			intentMenuPolyrhythms.putExtra("containerPolyrhythms",containerPolyrhythms);
			startActivityForResult(intentMenuPolyrhythms,ID_MENU_POLYRHYTHMS);
			return true;
		case ID_MENU_STRUCTURES:
			Intent intentMenuStructures = new Intent(Metronome.this, MenuStructures.class);
			intentMenuStructures.putExtra("containerStructures",containerStructures);
			startActivityForResult(intentMenuStructures,ID_MENU_STRUCTURES);
			return true;
		case ID_MENU_SETTINGS:
			Intent intentMenuSettings = new Intent(Metronome.this, MenuSettings.class);
			startActivityForResult(intentMenuSettings,ID_MENU_SETTINGS);
			return true;
		case ID_MENU_ABOUT:
			Intent intentMenuAbout = new Intent(Metronome.this, MenuAbout.class);
			startActivity(intentMenuAbout);
			return true;
		default:
			return super.onOptionsItemSelected(item);
	}
	}
	/** And grab back edited data. MenuSettings won't return any intent. */
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		super.onActivityResult(requestCode, resultCode, data);
		switch (requestCode) {
			case ID_MENU_RANDOMMUTE:
				if(resultCode == 1) {
					containerRandomMute = (ContainerRandomMute)data.getSerializableExtra("containerRandomMute");
					spinnerMode.setSelection(1);//Select randommute
				}
			return;
			case ID_MENU_POLYRHYTHMS:
				if(resultCode == 1) {
					containerPolyrhythms = (ContainerPolyrhythms)data.getSerializableExtra("containerPolyrhythms");
					int polyrhythmsCount = containerPolyrhythms.rhythms.length;
					if (polyrhythmsCount != 0)
						spinnerMode.setSelection(2);//Select polyrhythms
					else
						spinnerMode.setSelection(CLICKER_TASK);//In case empty polyrhythms
				}
			return;
			case ID_MENU_STRUCTURES:
				if(resultCode == 1) {
					containerStructures = (ContainerStructures)data.getSerializableExtra("containerStructures");
					int structuresCount = containerStructures.structures.length;
					listMode.clear();
					listMode.add(getResources().getString(R.string.listMode_normal));
					listMode.add(getResources().getString(R.string.listMode_randomMute));
					listMode.add(getResources().getString(R.string.listMode_polyrhythms));
					if (structuresCount != 0) {
						spinnerMode.setSelection(3);//Selection 1st structure
						for(int i=1;i<structuresCount+1;i++)
							listMode.add(String.format(getResources().getString(R.string.listMode_structure),i));
					}
					else
						spinnerMode.setSelection(CLICKER_TASK);//In case the previously selected structure has been deleted.
				}
			return;
			case ID_MENU_SETTINGS:
				if (resultCode == 1) {
					if (getSharedPreferences("settings", 0).getBoolean("isScreenAlwaysOn",false) == true)
						getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
					else
						getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
				}
			return;
		}
	}

	/* Buttons Handlers. */
	public void onTapIn (View v)
	{// If using SystemClock.elapsedRealtimeNanos() (API 17), you need to adapt marked lines.
		long delay, currentTapIn;
		double bpm;
		
		currentTapIn = SystemClock.elapsedRealtime();//<--
		delay = currentTapIn - lastTapIn;
		lastTapIn = currentTapIn;

		bpm = 60000./delay;//<--60000000000/delay if nanotime
		editTextBpm.setText(String.valueOf(bpm));
		//We don't do the "clear focus dance" (see onStartStop) do avoid bloat.
	}
	
	/** Toggle the clicker depending on the current state \var mode */
	public void onStartStop (View v)
	{
	if (mode == NO_TASK) {//Go!
		/** UI stuff */
		String stringBpm = editTextBpm.getText().toString();
		String stringBar = editTextBar.getText().toString();
		if (stringBpm.matches("") || stringBar.matches("")) {
			Toast.makeText(getApplicationContext(),R.string.toastEnterSomeValue,Toast.LENGTH_SHORT).show();
			return;
		}
		
		double bpm = Double.parseDouble(stringBpm);
		double bar = Double.parseDouble(stringBar);
		if (bpm <= 0) {
			Toast.makeText(getApplicationContext(),R.string.toastBpmCantBeZero,Toast.LENGTH_SHORT).show();
			return;
		}

		imageButtonStartStop.setImageResource(android.R.drawable.ic_media_pause);
		imageButtonStartStop.setBackgroundColor(0xFFFF0000);
		//Clear focus dance
		//Make the keyboard disappear
		editTextBpm.setEnabled(false);
		editTextBpm.setEnabled(true);
		//Clear the cursor. So the nect time we touch it, it will selectAll. clearFocus() actually move focus to the first focusable view (that used to be editTextBpm) so editTextBpm would be focused each time something called clearFocus() (exactly what we want to avoid!!). By setting the parent layout focusable(intouchmode), the focus is properly removed (moved to the layout).
		editTextBpm.clearFocus();
		editTextBar.setEnabled(false);
		editTextBar.setEnabled(true);
		editTextBar.clearFocus();

		/** Clicker Stuff */
		mode = spinnerMode.getSelectedItemPosition();
		if (mode == CLICKER_TASK) {//Normal Clicker
			clicker = new Clicker(bpm,bar,getApplicationContext());
			clicker.start();
		}
		else if (mode == RANDOMMUTE_TASK) {
			clickerRandomMute = new ClickerRandomMute(containerRandomMute,bpm,bar,getApplicationContext());
			clickerRandomMute.start();
		}
		else if (mode == POLYRHYTHMS_TASK) {//Polyrhythms Clicker
			clickerPolyrhythms = new ClickerPolyrhythms(containerPolyrhythms,bpm,bar,getApplicationContext());
			clickerPolyrhythms.start();
		}
		else {//Structure Clicker
			clickerStructure = new ClickerStructure(containerStructures,mode-3,bpm,bar,getApplicationContext());
			clickerStructure.start();
		}
	}
	else {//Stop!
		imageButtonStartStop.setImageResource(android.R.drawable.ic_media_play);
		imageButtonStartStop.setBackgroundColor(0xFF00FF00);
		//Clear focus
		editTextBpm.setEnabled(false);
		editTextBpm.setEnabled(true);
		editTextBpm.clearFocus();
		editTextBar.setEnabled(false);
		editTextBar.setEnabled(true);
		editTextBar.clearFocus();

		
		//The spinnerMode is only read when starting so even if we changed the spinner selection while clicking, mode is still unchanged on stop. That's why we are able to determine the class to shutdown between clicker or clickerStructure.
		if (mode == CLICKER_TASK) {
			clicker.stop();
		}
		else if (mode == RANDOMMUTE_TASK) {
			clickerRandomMute.stop();
		}
		else if (mode == POLYRHYTHMS_TASK) {
			clickerPolyrhythms.stop();
		}
		else {
			clickerStructure.stop();
			containerStructures.resetAudio();//Or else an eventual bpm change wont be computed.
		}
		mode = NO_TASK;
	}

	return;
	}

	public void onQuit (View v)
	{	
		SharedPreferences settings = getSharedPreferences("settings", 0);
		SharedPreferences.Editor editor = settings.edit();
		editor.putFloat("bpm",Float.parseFloat(editTextBpm.getText().toString()));
		editor.putFloat("bar",Float.parseFloat(editTextBar.getText().toString()));
		editor.putInt("listMode",spinnerMode.getSelectedItemPosition());
		editor.commit();
		finish();
		System.exit(0);
	}
}
